-- name: [CS] Big the Cat
-- description: Big The Cat must rescue froggy!\n\n\\#ff7777\\This Pack requires Character Select\nto use as a Library!

--[[
    API Documentation for Character Select can be found below:
    https://github.com/Squishy6094/character-select-coop/wiki/API-Documentation

    Use this if you're curious on how anything here works >v<
]]

define_custom_obj_fields(
    {
        oOwner = 'u32',
        oHitByShockwave = 'u32',
    }
)


local TEXT_MOD_NAME = "Big The Cat"
ACT_BIG_GLIDE = allocate_mario_action(ACT_GROUP_AIRBORNE | ACT_FLAG_ALLOW_VERTICAL_WIND_ACTION | ACT_FLAG_MOVING)
ACT_BIG_BELLYFLOP =
    allocate_mario_action(ACT_GROUP_AIRBORNE | ACT_FLAG_AIR | ACT_FLAG_ATTACKING | ACT_FLAG_ALLOW_VERTICAL_WIND_ACTION)
local bigcat = _G.charSelect.character_get_number_from_string("Big The Cat")
---@param o Object
local umbrella_loop = function(o)
    local m = gMarioStates[o.oOwner]
    local action = m.action == ACT_HIT_FREEZE and m.actionArg or m.action

    o.oFaceAngleYaw = m.marioObj.header.gfx.angle.y
    o.oFaceAnglePitch = m.marioObj.header.gfx.angle.x
    o.oFaceAngleRoll = m.marioObj.header.gfx.angle.z

    o.header.gfx.node.flags = o.header.gfx.node.flags & ~GRAPH_RENDER_INVISIBLE

    if m.marioObj.header.gfx.node.flags & GRAPH_RENDER_INVISIBLE ~= 0 or m.marioObj.header.gfx.node.flags & GRAPH_RENDER_ACTIVE == 0 then
        o.header.gfx.node.flags = o.header.gfx.node.flags | GRAPH_RENDER_INVISIBLE
    end

    if ACT_BIG_GLIDE then
        if o.header.gfx.scale.x < 1 then
            obj_scale(o, o.header.gfx.scale.x + 0.25)
        end

        o.header.gfx.animInfo.animFrame = 0
        o.header.gfx.animInfo.animID = 10
    else
        if o.header.gfx.scale.x > 0 then
            obj_scale(o, o.header.gfx.scale.x - 0.25)
        else
            obj_mark_for_deletion(o)
            ACT_BIG_GLIDE = nil
        end

        o.oOpacity = 0
    end
end
-- Stops mod from loading if Character Select isn't on
if not _G.charSelectExists then
    djui_popup_create("\\#ffffdc\\\n"..TEXT_MOD_NAME.."\nRequires the Character Select Mod\nto use as a Library!\n\nPlease turn on the Character Select Mod\nand Restart the Room!", 6)
    return 0
end

local E_MODEL_BIGCAT = smlua_model_util_get_id("bigthecat_geo") -- Located in "actors"

local TEX_CUSTOM_LIFE_BIG = get_texture_info("biglife") -- Located in "textures"
local TEX_CUSTOM_frog_ICON = get_texture_info("froggycollect") -- Located in "textures"

-- All Located in "sound"
local VOICETABLE_BIGCAT = {
    [CHAR_SOUND_ATTACKED] = 'bighurt.ogg',
    [CHAR_SOUND_DOH] = 'dizzy.ogg',
    [CHAR_SOUND_DROWNING] = 'bigdie.ogg',
    [CHAR_SOUND_DYING] = 'bigdie.ogg',
    [CHAR_SOUND_GROUND_POUND_WAH] = 'laugh.ogg',
    [CHAR_SOUND_HAHA] = 'laugh.ogg',
    [CHAR_SOUND_HAHA_2] = 'laugh.ogg',
    [CHAR_SOUND_HERE_WE_GO] = 'froggy (2).ogg',
    [CHAR_SOUND_HOOHOO] = 'lets go.ogg',
    [CHAR_SOUND_MAMA_MIA] = 'biguh-oh.ogg',
    [CHAR_SOUND_OKEY_DOKEY] = 'I can.ogg',
    [CHAR_SOUND_ON_FIRE] = 'help.ogg',
    [CHAR_SOUND_OOOF] = 'bighurt (2).ogg',
    [CHAR_SOUND_OOOF2] = 'bighurt.ogg',
    [CHAR_SOUND_PUNCH_HOO] = 'lets go.ogg',
    [CHAR_SOUND_PUNCH_WAH] = 'laugh.ogg',
    [CHAR_SOUND_PUNCH_YAH] = 'bigyahoo.ogg',
    [CHAR_SOUND_SO_LONGA_BOWSER] = 'givefroggyback.ogg',
    [CHAR_SOUND_TWIRL_BOUNCE] = 'laugh.ogg',
    [CHAR_SOUND_WAAAOOOW] = 'biguh-oh.ogg',
    [CHAR_SOUND_WAH2] = 'ledgegrabbig.ogg',
    [CHAR_SOUND_WHOA] = 'ledgegrabbig.ogg',
    [CHAR_SOUND_YAHOO] = 'hoo-hee.ogg',
    [CHAR_SOUND_YAHOO_WAHA_YIPPEE] = 'laugh.ogg',
    [CHAR_SOUND_YAH_WAH_HOO] = 'jumpingbig.ogg',
    [CHAR_SOUND_YAWNING] = 'fishing.ogg',
}

local E_MODEL_frog_STAR = smlua_model_util_get_id("froggy_geo") -- Located in "actors"

local PALETTE_CHAR = {
    [PANTS]  = "ffffff",
    [SHIRT]  = "ffffff",
    [GLOVES] = "ffffff",
    [SHOES]  = "ffffff",
    [HAIR]   = "ffffff",
    [SKIN]   = "ffffff",
    [CAP]    = "ffffff",
}

--[[
-- All Located in "textures"
local healthMeter = {
    label = {
        left = get_texture_info("template-hp-back-left"),
        right = get_texture_info("template-hp-back-right"),
    },
    pie = {
        [1] = get_texture_info("template-hp-pie-1"),
        [2] = get_texture_info("template-hp-pie-2"),
        [3] = get_texture_info("template-hp-pie-3"),
        [4] = get_texture_info("template-hp-pie-4"),
        [5] = get_texture_info("template-hp-pie-5"),
        [6] = get_texture_info("template-hp-pie-6"),
        [7] = get_texture_info("template-hp-pie-7"),
        [8] = get_texture_info("template-hp-pie-8"),
    }
}
]]

--[[
    Note: If there are some functionalities you don't care
    to add such as palettes, you can freely remove the function
    for it, the only function you require is character_add
]]

local function act_big_glide(m)
    if gPlayerSyncTable[m.playerIndex].bigmoves == true then


    -- apply movement when using action
    common_air_action_step(m, ACT_JUMP_LAND, 0, AIR_STEP_NONE)
    smlua_anim_util_set_animation(m.marioObj, "Big_Glide")
    

    -- setup when action starts (horizontal speed and voiceline)
    if m.actionTimer == 0 then
        if m.forwardVel > 20 then
            mario_set_forward_vel(m, 20)
        end
        play_character_sound(m, CHAR_SOUND_HELLO)
    end

    -- block all y movement when using the move
    m.vel.y = -10
    set_mario_particle_flags(m, PARTICLE_SPARKLES, 0)

    -- avoid issue with flying and then make the hover end after 2 secs or when stopping holding the button
    if m.prevAction ~= ACT_TRIPLE_JUMP and (m.flags & MARIO_WING_CAP) ~= 0 then
        if m.actionTimer >= 50 or (m.controller.buttonDown & A_BUTTON) == 0 then
            set_mario_action(m, ACT_FREEFALL, 0)
        end
    else
        if m.actionTimer >= 50 or (m.controller.buttonDown & A_BUTTON) == 0 then
            set_mario_action(m, ACT_FREEFALL, 0)
        end
    end
end
end

    
local CSloaded = false
local function on_character_select_load()
    bigcat = _G.charSelect.character_add("Big The Cat", {"Big The Cat arrived in the mushroom kingdom on order of his pet frog.", "The frog assumed the toads trapped in the castle where ACTUAL toads", " whom the frogs have an alliance with"}, "SEGA/Chaorrin", {r = 255, g = 00, b = 255}, E_MODEL_BIGCAT, CT_MARIO, TEX_CUSTOM_LIFE_BIG)
   -- _G.charSelect.character_add_caps(E_MODEL_CUSTOM_MODEL, nil)
    _G.charSelect.character_add_voice(E_MODEL_BIGCAT, VOICETABLE_BIGCAT)
    _G.charSelect.character_add_celebration_star(E_MODEL_BIGCAT, E_MODEL_frog_STAR, TEX_CUSTOM_frog_ICON)
   -- _G.charSelect.character_add_palette_preset(E_MODEL_CUSTOM_MODEL, PALETTE_CHAR)
    --_G.charSelect.character_add_health_meter(CT_CHAR, healthMeter)

    CSloaded = true
end

local function on_character_sound(m, sound)
    if not CSloaded then return end
    if _G.charSelect.character_get_voice(m) == VOICETABLE_BIGCAT then return _G.charSelect.voice.sound(m, sound) end
end

local function on_character_snore(m)
    if not CSloaded then return end
    if _G.charSelect.character_get_voice(m) == VOICETABLE_BIGCAT then return _G.charSelect.voice.snore(m) end
end

hook_event(HOOK_ON_MODS_LOADED, on_character_select_load)
hook_event(HOOK_CHARACTER_SOUND, on_character_sound)
hook_event(HOOK_MARIO_UPDATE, on_character_snore)

local function mario_update(m)
    if m.playerIndex == 0 then
        if bigcat == _G.charSelect.character_get_current_number() then
            gPlayerSyncTable[m.playerIndex].bigmoves= true
        else
            gPlayerSyncTable[m.playerIndex].bigmoves = false
        end
    end

    if gPlayerSyncTable[m.playerIndex].bigmoves then




    if  m.action == ACT_HOLD_WALKING or
       m.action == ACT_HOLD_HEAVY_WALKING then 
        m.action = ACT_HOLD_WALKING 
     
local speedThreshold = 35
m.forwardVel = math.max(m.forwardVel,speedThreshold)

end
end
   


        if (m.controller.buttonDown & A_BUTTON) ~= 0 and m.vel.y < -15 and m.prevAction ~= ACT_BIG_GLIDE and  (
        m.action == ACT_JUMP or
        m.action == ACT_DOUBLE_JUMP or
        m.action == ACT_TRIPLE_JUMP or
        m.action == ACT_LONG_JUMP or
        m.action == ACT_BACKFLIP or
        m.action == ACT_SIDE_FLIP or
        m.action == ACT_WALL_KICK_AIR) then
            if gPlayerSyncTable[m.playerIndex].bigmoves == true then

             set_mario_action(m, ACT_BIG_GLIDE, 0)
             print('spawning paddle: ', id_bhvBreakableBox, E_MODEL_BIG_PADDLE, COL_BIG_PADDLE)
             -- spawn paddle
             spawn_sync_object(    id_bhvBigumbrella,   E_MODEL_BIG_PADDLE, m.pos.x, m.pos.y, m.pos.z, function(o)
                 o.oOwner = m.playerIndex 
                 o.hookRender = 1
             end)
             local function act_big_glide(m)

                -- apply movement when using action
                common_air_action_step(m, ACT_JUMP_LAND, 0, AIR_STEP_NONE)
                smlua_anim_util_set_animation(m.marioObj, "Big_Glide")
            
                -- setup when action starts (horizontal speed and voiceline)
                if m.actionTimer == 0 then
                    if m.forwardVel > 20 then
                        mario_set_forward_vel(m, 20)
                    end
                    play_character_sound(m, CHAR_SOUND_HELLO)
                end
            
                -- block all y movement when using the move
                m.vel.y = -10
                set_mario_particle_flags(m, PARTICLE_SPARKLES, 0)
            
                -- avoid issue with flying and then make the hover end after 2 secs or when stopping holding the button
                if m.prevAction ~= ACT_TRIPLE_JUMP and (m.flags & MARIO_WING_CAP) ~= 0 then
                    if m.actionTimer >= 50 or (m.controller.buttonDown & A_BUTTON) == 0 then
                        set_mario_action(m, ACT_FREEFALL, 0)
                    end
                else
                    if m.actionTimer >= 50 or (m.controller.buttonDown & A_BUTTON) == 0 then
                        set_mario_action(m, ACT_FREEFALL, 0)
                    
                end
            end
        end end
    end
       if (m.controller.buttonDown & Y_BUTTON) ~= 0 and m.vel.y < -15 and m.prevAction then
        if gPlayerSyncTable[m.playerIndex].bigmoves == true then
           set_mario_action(m, ACT_BIG_BELLYFLOP, 1)
               
                 
                play_character_sound(m, CHAR_SOUND_HAHA)
                     else

                     end



                     end
                
                        
                        
                    end
          
            local function act_big_flop(m)
                
                common_air_action_step(m, ACT_JUMP_LAND, 1, AIR_STEP_NONE)
                smlua_anim_util_set_animation(m.marioObj, "Bigbellyflop")
                
                
                local stepResult = perform_ground_step(m)
                if stepResult == GROUND_STEP_NONE then
                        m.flags = m.flags | MARIO_KICKING
                        apply_slope_decel(m, .6)
                        apply_slope_accel(m)
            
                     if m.vel.x == 0 or m.vel.z == 0  then
                        
                   set_mario_action(m, ACT_GROUND_POUND_LAND, 0)
             
            
                m.particleFlags = m.particleFlags | PARTICLE_VERTICAL_STAR
            
                
            
            end
        end
    end



        
    
hook_event(HOOK_MARIO_UPDATE, mario_update)
hook_mario_action(ACT_BIG_GLIDE, act_big_glide)
hook_mario_action(ACT_BIG_BELLYFLOP, act_big_flop)




